@@ -13,42 +13,39 @@
 #' @param integer Logical, if `TRUE`, coerces the output to integer. Default: `FALSE`
 #' @return A numeric vector of the same length as x, but with its values rescaled between `new.min` and `new.max.`
 #' @examples
-#' if(interactive()){
 #'
-#'  out <- rescale_vector(
-#'    x = rnorm(100),
-#'    new.min = 0,
-#'    new.max = 100,
-#'    integer = TRUE
-#'    )
-#'    out
+#' y <- rescale_vector(
+#'   x = rnorm(100),
+#'   new.min = 0,
+#'   new.max = 100,
+#'   integer = TRUE
+#' )
+#' y
 #'
-#' }
 #' @rdname rescale_vector
+#' @family utilities
 #' @export
-rescale_vector <- function(x = NULL,
-                           new.min = 0,
-                           new.max = 1,
-                           integer = FALSE){
-
-  if(is.null(x) | !is.vector(x) | !is.numeric(x)){
+rescale_vector <- function(
+  x = NULL,
+  new.min = 0,
+  new.max = 1,
+  integer = FALSE
+) {
+  if (is.null(x) || !is.vector(x) || !is.numeric(x)) {
     stop("x must be a numeric vector.")
   }
 
   #data extremes
-  old.min = min(x)
-  old.max = max(x)
-
+  old.min <- min(x)
+  old.max <- max(x)
 
   #scaling
-  x = ((x - old.min) / (old.max - old.min)) * (new.max - new.min) + new.min
-
+  x <- ((x - old.min) / (old.max - old.min)) * (new.max - new.min) + new.min
 
   #to integer
-  if(integer == TRUE){
-    x = floor(x)
+  if (integer) {
+    x <- floor(x)
   }
 
   x
-
 }
