@@ -62,57 +62,48 @@
 #' }
 #' Once ranking procedure is completed, an algorithm is used to select the minimal subset of spatial predictors that reduce the most the Moran's I of the residuals: for each new spatial predictor introduced in the model, the Moran's I of the residuals, it's p-value, a binary version of the p-value (0 if < 0.05 and 1 if >= 0.05), the R-squared of the model, and a penalization linear with the number of spatial predictors introduced (computed as `(1 / total spatial predictors) * introduced spatial predictors`) are rescaled between 0 and 1. Then, the optimization criteria is computed as `max(1 - Moran's I, p-value binary) + (weight.r.squared * R-squared) - (weight.penalization.n.predictors * penalization)`. The predictors from the first one to the one with the highest optimization criteria are then selected as the best ones in reducing the spatial correlation of the model residuals, and used along with `data` to fit the final spatial model.
 #' @examples
-#' if(interactive()){
 #'
-#'  #loading example data
-#'  data(distance_matrix)
-#'  data(plant_richness_df)
+#' if (interactive()) {
+#'   data(
+#'     plants_df,
+#'     plants_response,
+#'     plants_predictors,
+#'     plants_distance,
+#'     plants_rf
+#'   )
 #'
-#'  #names of the response and predictors
-#'  dependent.variable.name <- "richness_species_vascular"
-#'  predictor.variable.names <- colnames(plant_richness_df)[5:21]
+#'   #subset to speed up example
+#'   idx <- 1:100
+#'   plants_df <- plants_df[idx, ]
+#'   plants_distance <- plants_distance[idx, idx]
 #'
-#'  #hengl
-#'  model <- rf_spatial(
-#'    data = plant_richness_df,
-#'    dependent.variable.name = dependent.variable.name,
-#'    predictor.variable.names = predictor.variable.names,
-#'    distance.matrix = distance_matrix,
-#'    distance.thresholds = 0,
-#'    method = "hengl",
-#'    n.cores = 1
-#'  )
+#'   #fit spatial model from scratch
+#'   m_spatial <- rf_spatial(
+#'     data = plants_df,
+#'     dependent.variable.name = plants_response,
+#'     predictor.variable.names = plants_predictors,
+#'     distance.matrix = plants_distance,
+#'     distance.thresholds = c(100, 1000, 2000),
+#'     method = "mem.moran.sequential",
+#'     ranger.arguments = list(num.trees = 30),
+#'     n.cores = 1
+#'   )
 #'
-#'  #mem.moran.sequential
-#'  model <- rf_spatial(
-#'    data = plant_richness_df,
-#'    dependent.variable.name = dependent.variable.name,
-#'    predictor.variable.names = predictor.variable.names,
-#'    distance.matrix = distance_matrix,
-#'    distance.thresholds = 0,
-#'    method = "mem.moran.sequential",
-#'    n.cores = 1
-#'  )
+#'   plot_residuals_diagnostics(m_spatial)
 #'
-#'  #fitting an rf_spatial model from an rf model
-#'  rf.model <- rf(
-#'    data = plant_richness_df,
-#'    dependent.variable.name = "richness_species_vascular",
-#'    predictor.variable.names = colnames(plant_richness_df)[5:21],
-#'    distance.matrix = distance_matrix,
-#'    distance.thresholds = 0,
-#'    n.cores = 1,
-#'    verbose = FALSE
-#'  )
-#'  rf.model$spatial.correlation.residuals$plot
-#'
-#'  #spatial version of the rf model
-#'  rf.spatial <- rf_spatial(model = rf.model)
-#'  rf.spatial$spatial.correlation.residuals$plot
+#'   #optimization of MEM selection
+#'   plot_optimization(m_spatial)
 #'
+#'   #from non-spatial to spatial model
+#'   m_spatial <- rf_spatial(
+#'     model = plants_rf
+#'     )
 #'
 #' }
+#'
+#'
 #' @rdname rf_spatial
+#' @family main_models
 #' @export
 rf_spatial <- function(
   model = NULL,
@@ -143,8 +134,7 @@ rf_spatial <- function(
   verbose = TRUE,
   n.cores = parallel::detectCores() - 1,
   cluster = NULL
-){
-
+) {
   #declaring variables
   moran.i <- NULL
   variable <- NULL
@@ -173,48 +163,40 @@ rf_spatial <- function(
   #####################################
 
   #if model is not provided
-  if(is.null(model)){
-
+  if (is.null(model)) {
     #stopping if no data
-    if(is.null(data)){
-
+    if (is.null(data)) {
       stop("The argument 'data' is missing.")
-
     } else {
-
       #coerce to data frame if tibble
-      if(inherits(data, "tbl_df") | inherits(data, "tbl")){
+      if (inherits(data, "tbl_df") || inherits(data, "tbl")) {
         data <- as.data.frame(data)
       }
-
     }
 
     #coerce to data frame if tibble
-    if(inherits(xy, "tbl_df") | inherits(xy, "tbl")){
+    if (inherits(xy, "tbl_df") || inherits(xy, "tbl")) {
       xy <- as.data.frame(xy)
     }
 
     #stopping if no distance matrix
-    if(is.null(distance.matrix)){
+    if (is.null(distance.matrix)) {
       stop("The argument 'distance.matrix' is missing.")
     }
     #stopping if no dependent.variable.name
-    if(is.null(dependent.variable.name)){
+    if (is.null(dependent.variable.name)) {
       stop("The argument 'dependent.variable.name' is missing.")
     }
     #stopping if no predictor.variable.names
-    if(is.null(predictor.variable.names)){
+    if (is.null(predictor.variable.names)) {
       stop("The argument 'predictor.variable.names' is missing.")
     } else {
       #predictor.variable.names comes from auto_vif or auto_cor
-      if(inherits(predictor.variable.names, "variable_selection")){
+      if (inherits(predictor.variable.names, "variable_selection")) {
         predictor.variable.names <- predictor.variable.names$selected.variables
       }
     }
 
-    #controlling num.threads
-    ranger.arguments$num.threads <- n.cores
-
     #fitting non-spatial model
     model <- rf(
       data = data,
@@ -226,63 +208,31 @@ rf_spatial <- function(
       ranger.arguments = ranger.arguments,
       scaled.importance = FALSE,
       seed = seed,
+      n.cores = n.cores,
       verbose = FALSE
     )
-
-    } else {
-
-      #getting arguments from model
-      ranger.arguments <- model$ranger.arguments
-      data <- ranger.arguments$data
-      dependent.variable.name <- ranger.arguments$dependent.variable.name
-      predictor.variable.names <- ranger.arguments$predictor.variable.names
-      distance.matrix <- ranger.arguments$distance.matrix
-      if(is.null(distance.matrix)){
-        stop("The argument 'distance.matrix' is missing.")
-      }
-      distance.thresholds <- ranger.arguments$distance.thresholds
-      if(is.null(distance.thresholds)){
-        distance.thresholds <- default_distance_thresholds(distance.matrix = distance.matrix)
-      }
-      scaled.importance <- ranger.arguments$scaled.importance
-      seed <- model$ranger.arguments$seed
-
-      #getting cluster from model if "cluster" is not provided
-      if(is.null(cluster) & "cluster" %in% names(model)){
-          cluster <- model$cluster
-      }
-
-
-    }
-
-  #CLUSTER SETUP
-  #cluster is provided
-  if(!is.null(cluster)){
-
-    #n.cores <- NULL
-    n.cores <- NULL
-
-    #flat to not stop cluster after execution
-    stop.cluster <- FALSE
-
   } else {
-
-    #creates and registers cluster
-    cluster <- parallel::makeCluster(
-      n.cores,
-      type = "PSOCK"
-    )
-
-    #flag to stop cluster
-    stop.cluster <- TRUE
-
+    #getting arguments from model
+    ranger.arguments <- model$ranger.arguments
+    data <- ranger.arguments$data
+    dependent.variable.name <- ranger.arguments$dependent.variable.name
+    predictor.variable.names <- ranger.arguments$predictor.variable.names
+    distance.matrix <- ranger.arguments$distance.matrix
+    if (is.null(distance.matrix)) {
+      stop("The argument 'distance.matrix' is missing.")
+    }
+    distance.thresholds <- ranger.arguments$distance.thresholds
+    if (is.null(distance.thresholds)) {
+      distance.thresholds <- default_distance_thresholds(
+        distance.matrix = distance.matrix
+      )
+    }
+    scaled.importance <- ranger.arguments$scaled.importance
+    seed <- model$ranger.arguments$seed
   }
 
-  #registering cluster
-  doParallel::registerDoParallel(cl = cluster)
-
   #reference moran's I for selection of spatial predictors
-  if(!is.null(model$residuals$autocorrelation$max.moran)){
+  if (!is.null(model$residuals$autocorrelation$max.moran)) {
     reference.moran.i <- model$residuals$autocorrelation$max.moran
   } else {
     reference.moran.i <- 1
@@ -294,21 +244,20 @@ rf_spatial <- function(
     dplyr::filter(interpretation == "Positive spatial correlation")
 
   #if residuals are not autocorrelated, return original model
-  if(nrow(model.moran.i) == 0){
-
-    if(verbose == TRUE){
-      message("The model residuals are not spatially correlated, there is no need to fit a spatial model")
+  if (nrow(model.moran.i) == 0) {
+    if (verbose) {
+      message(
+        "The model residuals are not spatially correlated, there is no need to fit a spatial model"
+      )
     }
-
-    return(model)
-
   } else {
     #if residuals are autocorrelated
 
-    if(verbose == TRUE){
-      message("The model residuals are spatially correlated, fitting a spatial model.")
+    if (verbose) {
+      message(
+        "The model residuals are spatially correlated, fitting a spatial model."
+      )
     }
-
   }
 
   #getting distance thresholds with positive AC
@@ -319,61 +268,63 @@ rf_spatial <- function(
   #########################################################
 
   #HENGL
-  if(method %in% c(
-    "hengl",
-    "hengl.moran.sequential",
-    "hengl.effect.sequential",
-    "hengl.effect.recursive"
-  )
-  ){
-
+  if (
+    method %in%
+      c(
+        "hengl",
+        "hengl.moran.sequential",
+        "hengl.effect.sequential",
+        "hengl.effect.recursive"
+      )
+  ) {
     #change name of distance matrix
     spatial.predictors.df <- distance.matrix
-    colnames(spatial.predictors.df) <- rownames(spatial.predictors.df) <- paste0(
+    colnames(spatial.predictors.df) <- rownames(
+      spatial.predictors.df
+    ) <- paste0(
       "spatial_predictor_",
       seq(
         1,
         ncol(distance.matrix)
-        )
       )
+    )
     spatial.predictors.selected <- colnames(spatial.predictors.df)
 
-    if(verbose == TRUE){
+    if (verbose) {
       message("Using the distance matrix columns as spatial predictors.")
     }
-
-
   }
 
   #PCA
-  if(method %in% c(
-    "pca.moran.sequential",
-    "pca.effect.sequential",
-    "pca.effect.recursive"
-  )
-  ){
-
+  if (
+    method %in%
+      c(
+        "pca.moran.sequential",
+        "pca.effect.sequential",
+        "pca.effect.recursive"
+      )
+  ) {
     #computing pca factors for pca methods
     spatial.predictors.df <- pca_multithreshold(
       distance.matrix = distance.matrix,
-      distance.thresholds =  distance.thresholds.with.ac,
+      distance.thresholds = distance.thresholds.with.ac,
       max.spatial.predictors = max.spatial.predictors
     )
 
-    if(verbose == TRUE){
+    if (verbose) {
       message("Using PCA factors of the distance matrix as spatial predictors.")
     }
-
   }
 
   #MEM
-  if(method %in% c(
-    "mem.moran.sequential",
-    "mem.effect.sequential",
-    "mem.effect.recursive"
-  )
-  ){
-
+  if (
+    method %in%
+      c(
+        "mem.moran.sequential",
+        "mem.effect.sequential",
+        "mem.effect.recursive"
+      )
+  ) {
     #computing mem
     spatial.predictors.df <- mem_multithreshold(
       distance.matrix = distance.matrix,
@@ -381,50 +332,68 @@ rf_spatial <- function(
       max.spatial.predictors = max.spatial.predictors
     )
 
-    if(verbose == TRUE){
+    if (verbose) {
       message("Using Moran's Eigenvector Maps as spatial predictors.")
     }
-
   }
 
   #RANKING SPATIAL PREDICTORS
   ###########################################################
 
   #SELECTING RANKING METHOD
-  if(method %in% "hengl"){
+  if (method %in% "hengl") {
     ranking.method <- NULL
   }
 
-  if(method %in% c(
-    "hengl.moran.sequential",
-    "pca.moran.sequential",
-    "mem.moran.sequential"
-  )
-  ){
+  if (
+    method %in%
+      c(
+        "hengl.moran.sequential",
+        "pca.moran.sequential",
+        "mem.moran.sequential"
+      )
+  ) {
     ranking.method <- "moran"
-    if(verbose == TRUE){
+    if (verbose) {
       message("Ranking spatial predictors by their Moran's I.")
     }
   }
 
-  if(method %in% c(
-    "hengl.effect.sequential",
-    "hengl.effect.recursive",
-    "pca.effect.sequential",
-    "pca.effect.recursive",
-    "mem.effect.sequential",
-    "mem.effect.recursive"
-  )
-  ){
+  if (
+    method %in%
+      c(
+        "hengl.effect.sequential",
+        "hengl.effect.recursive",
+        "pca.effect.sequential",
+        "pca.effect.recursive",
+        "mem.effect.sequential",
+        "mem.effect.recursive"
+      )
+  ) {
     ranking.method <- "effect"
-    if(verbose == TRUE){
-      message("Ranking spatial predictors by how much they reduce Moran's I of the model residuals.")
+    if (verbose) {
+      message(
+        "Ranking spatial predictors by how much they reduce Moran's I of the model residuals."
+      )
     }
   }
 
-  #ranking
-  if(!is.null(ranking.method)){
+  # Handle model$cluster if present
+  cluster.from.model <- FALSE
+  if (
+    !inherits(x = cluster, what = "cluster") &&
+      inherits(x = model$cluster, what = "cluster")
+  ) {
+    cluster <- model$cluster
+    cluster.from.model <- TRUE
+  }
 
+  #CLUSTER SETUP
+  parallel_config <- setup_parallel_execution(cluster, n.cores)
+  on.exit(parallel_config$cleanup(), add = TRUE)
+
+  #ranking
+  if (!is.null(ranking.method)) {
     #removes redundant spatial predictors
     spatial.predictors.df <- filter_spatial_predictors(
       data = data,
@@ -433,6 +402,16 @@ rf_spatial <- function(
       cor.threshold = 0.50
     )
 
+    #check if any spatial predictors remain after filtering
+    if (ncol(spatial.predictors.df) == 0) {
+      if (verbose) {
+        message(
+          "No spatial predictors remain after filtering. Returning non-spatial model."
+        )
+      }
+      return(model)
+    }
+
     #ranking spatial predictors
     spatial.predictors.ranking <- rank_spatial_predictors(
       data = data,
@@ -446,26 +425,26 @@ rf_spatial <- function(
       reference.moran.i = reference.moran.i,
       verbose = FALSE,
       n.cores = n.cores,
-      cluster = cluster
+      cluster = parallel_config$cluster
     )
-
   }
 
   #SELECTING SPATIAL PREDICTORS (if method is not "hengl")
   ######################################################
 
   #SEQUENTIAL SELECTION OF SPATIAL PREDICTORS
-  if(method %in% c(
-    "hengl.moran.sequential",
-    "mem.moran.sequential",
-    "pca.moran.sequential",
-    "hengl.effect.sequential",
-    "mem.effect.sequential",
-    "pca.effect.sequential"
-  )
-  ){
-
-    if(verbose == TRUE){
+  if (
+    method %in%
+      c(
+        "hengl.moran.sequential",
+        "mem.moran.sequential",
+        "pca.moran.sequential",
+        "hengl.effect.sequential",
+        "mem.effect.sequential",
+        "pca.effect.sequential"
+      )
+  ) {
+    if (verbose) {
       message("Sequential selection of spatial predictors.")
     }
 
@@ -483,24 +462,25 @@ rf_spatial <- function(
       weight.penalization.n.predictors = weight.penalization.n.predictors,
       verbose = FALSE,
       n.cores = n.cores,
-      cluster = cluster
+      cluster = parallel_config$cluster
     )
 
     #names of the selected spatial predictors
-    spatial.predictors.selected <- as.character(spatial.predictors.selection$best.spatial.predictors)
-
+    spatial.predictors.selected <- as.character(
+      spatial.predictors.selection$best.spatial.predictors
+    )
   }
 
-
   #recursive SELECTION OF SPATIAL PREDICTORS
-  if(method %in% c(
-    "hengl.effect.recursive",
-    "pca.effect.recursive",
-    "mem.effect.recursive"
-  )
-  ){
-
-    if(verbose == TRUE){
+  if (
+    method %in%
+      c(
+        "hengl.effect.recursive",
+        "pca.effect.recursive",
+        "mem.effect.recursive"
+      )
+  ) {
+    if (verbose) {
       message("Recursive selection of spatial predictors.")
     }
 
@@ -517,22 +497,22 @@ rf_spatial <- function(
       weight.r.squared = weight.r.squared,
       weight.penalization.n.predictors = weight.penalization.n.predictors,
       n.cores = n.cores,
-      cluster = cluster
-
+      cluster = parallel_config$cluster
     )
 
     #broadcast spatial.predictors.selected
     spatial.predictors.selected <- spatial.predictors.selection$best.spatial.predictors
-
   }
 
-
   #FITTING SPATIAL MODEL
   ######################
 
   #subsetting spatial predictors
-  if(method != "hengl"){
-    spatial.predictors.df <- spatial.predictors.df[, spatial.predictors.selected]
+  if (method != "hengl") {
+    spatial.predictors.df <- spatial.predictors.df[,
+      spatial.predictors.selected,
+      drop = FALSE
+    ]
   }
 
   #prepare training data with best spatial predictors
@@ -561,6 +541,7 @@ rf_spatial <- function(
     xy = xy,
     ranger.arguments = ranger.arguments,
     seed = seed,
+    n.cores = n.cores,
     verbose = FALSE
   )
 
@@ -584,34 +565,29 @@ rf_spatial <- function(
   model.spatial$spatial$names <- spatial.predictors.selected
   model.spatial$spatial$spatial.predictors <- spatial.predictors.df
 
-  if(exists("spatial.predictors.selection")){
-
+  if (exists("spatial.predictors.selection")) {
     model.spatial$spatial$optimization <- spatial.predictors.selection$optimization
     model.spatial$spatial$plot <- plot_optimization(
       spatial.predictors.selection$optimization,
       verbose = FALSE
     )
-
   }
 
-  if(verbose == TRUE){
+  if (verbose) {
     message("Details about the spatial predictors stored in model$spatial.")
   }
 
-  if(verbose == TRUE){
+  if (verbose) {
     print(model.spatial)
     plot_importance(model.spatial)
     plot_residuals_diagnostics(model.spatial)
   }
 
-  #stopping cluster
-  if(stop.cluster == TRUE){
-    parallel::stopCluster(cl = cluster)
-  } else {
-    model.spatial$cluster <- cluster
+  # Store cluster in model if it was external (not internally created)
+  if (parallel_config$mode == "external_cluster" || cluster.from.model) {
+    model.spatial$cluster <- parallel_config$cluster
   }
 
   #return output
   model.spatial
-
 }
